<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Admin;

use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Style;
use Locr\WordPress\Plugin\LocrMaps\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the database.
 */
final class Database {
	/**
	 * This function drops all plugin tables.
	 */
	public static function drop_all_plugin_tables(): void {
		global $wpdb;

		// @phpstan-ignore method.nonObject
		$wpdb->query(
			// @phpstan-ignore method.nonObject
			$wpdb->prepare( 'DROP TABLE IF EXISTS %i CASCADE', self::get_icons_table_name() )
		);
		// @phpstan-ignore method.nonObject
		$wpdb->query(
			// @phpstan-ignore method.nonObject
			$wpdb->prepare( 'DROP TABLE IF EXISTS %i CASCADE', self::get_onlinemaps_table_name() )
		);
		// @phpstan-ignore method.nonObject
		$wpdb->query(
			// @phpstan-ignore method.nonObject
			$wpdb->prepare( 'DROP TABLE IF EXISTS %i CASCADE', self::get_onlinemap_locations_table_name() )
		);
	}

	/**
	 * This function checks if a foreign key exists.
	 *
	 * @param string $table_name The name of the table.
	 * @param string $fk_name The name of the foreign key.
	 */
	private static function foreign_key_exists( $table_name, $fk_name ): bool {
		global $wpdb;

		// @phpstan-ignore method.nonObject
		$result = $wpdb->get_var(
			// @phpstan-ignore method.nonObject
			$wpdb->prepare(
				'SELECT CONSTRAINT_NAME
					FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
					WHERE TABLE_NAME = %s AND CONSTRAINT_NAME = %s AND TABLE_SCHEMA = %s',
				$table_name,
				$fk_name,
				// @phpstan-ignore property.nonObject
				$wpdb->dbname
			)
		);

		return ! is_null( $result );
	}

	/**
	 * This function gets the charset collate.
	 */
	private static function get_charset_collate(): string {
		global $wpdb;
		// @phpstan-ignore method.nonObject, return.type
		return $wpdb->get_charset_collate();
	}

	/**
	 * This function gets the database engine of a table.
	 *
	 * @param string $table_name The name of the table.
	 */
	private static function get_db_engine_of_table( $table_name ): ?string {
		global $wpdb;

		// @phpstan-ignore method.nonObject, return.type
		return $wpdb->get_var(
			// @phpstan-ignore method.nonObject
			$wpdb->prepare(
				'SELECT ENGINE FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = %s AND TABLE_NAME = %s',
				// @phpstan-ignore property.nonObject
				$wpdb->dbname,
				$table_name
			)
		);
	}

	/**
	 * This function gets the name of the icons table.
	 */
	private static function get_icons_table_name(): string {
		global $wpdb;

		// @phpstan-ignore property.nonObject
		$table_prefix = $wpdb->prefix;
		if ( ! is_string( $table_prefix ) ) {
			$table_prefix = '';
		}
		return $table_prefix . Plugin::SLUG . '_icons';
	}

	/**
	 * This function gets the name of the onlinemaps table.
	 */
	private static function get_onlinemaps_table_name(): string {
		global $wpdb;

		// @phpstan-ignore property.nonObject
		$table_prefix = $wpdb->prefix;
		if ( ! is_string( $table_prefix ) ) {
			$table_prefix = '';
		}
		return $table_prefix . Plugin::SLUG . '_onlinemaps';
	}

	/**
	 * This function gets the name of the onlinemap locations table.
	 */
	private static function get_onlinemap_locations_table_name(): string {
		global $wpdb;

		// @phpstan-ignore property.nonObject
		$table_prefix = $wpdb->prefix;
		if ( ! is_string( $table_prefix ) ) {
			$table_prefix = '';
		}
		return $table_prefix . Plugin::SLUG . '_onlinemap_locations';
	}

	/**
	 * This function initializes the table for the icons.
	 */
	public static function init_icons_table(): void {
		$table_name      = self::get_icons_table_name();
		$charset_collate = self::get_charset_collate();

		$sql = "CREATE TABLE $table_name (
			id int NOT NULL AUTO_INCREMENT,
			icon_url varchar(255) NOT NULL UNIQUE,
			`name` varchar(255) UNIQUE,
			width int NOT NULL,
			height int NOT NULL,
			scale float NOT NULL DEFAULT 1,
			anchor varchar(32) NOT NULL DEFAULT 'center',
			PRIMARY KEY  (id)
		) $charset_collate;";

		// @phpstan-ignore requireOnce.fileNotFound
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		dbDelta( $sql );
	}

	/**
	 * This function initializes the table for the onlinemaps settings.
	 */
	public static function init_onlinemaps_table(): void {
		$table_name      = self::get_onlinemaps_table_name();
		$charset_collate = self::get_charset_collate();

		$sql = "CREATE TABLE $table_name (
			id int NOT NULL AUTO_INCREMENT,
			`name` varchar(255),
			style varchar(4) NOT NULL DEFAULT '" . Style::DEFAULT . "',
			width varchar(32) NOT NULL DEFAULT '',
			height varchar(32) NOT NULL DEFAULT '',
			border text NOT NULL DEFAULT '{}',
			PRIMARY KEY  (id)
		) $charset_collate;";

		// @phpstan-ignore requireOnce.fileNotFound
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		dbDelta( $sql );
	}

	/**
	 * This function initializes the table for the onlinemap locations settings.
	 */
	public static function init_onlinemap_locations_table(): void {
		$table_name      = self::get_onlinemap_locations_table_name();
		$charset_collate = self::get_charset_collate();

		$sql = "CREATE TABLE {$table_name} (
			id int NOT NULL AUTO_INCREMENT,
			onlinemap_id int NOT NULL,
			icon_id int NOT NULL,
			latitude double NOT NULL,
			longitude double NOT NULL,
			`address` JSON NOT NULL DEFAULT '{}',
			PRIMARY KEY (id)
		) $charset_collate;";

		// @phpstan-ignore requireOnce.fileNotFound
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		dbDelta( $sql );

		$db_engine = self::get_db_engine_of_table( self::get_onlinemap_locations_table_name() );
		if ( 'InnoDB' === $db_engine ) {
			global $wpdb;

			if ( ! self::foreign_key_exists( $table_name, 'fk_onlinemap_id' ) ) {
				// @phpstan-ignore method.nonObject
				$wpdb->query(
					// @phpstan-ignore method.nonObject
					$wpdb->prepare(
						'ALTER TABLE %i ADD CONSTRAINT `fk_onlinemap_id` FOREIGN KEY (`onlinemap_id`) REFERENCES %i(`id`) ON DELETE CASCADE ON UPDATE CASCADE',
						self::get_onlinemap_locations_table_name(),
						self::get_onlinemaps_table_name()
					)
				);
			}
			if ( ! self::foreign_key_exists( $table_name, 'fk_icon_id' ) ) {
				// @phpstan-ignore method.nonObject
				$wpdb->query(
					// @phpstan-ignore method.nonObject
					$wpdb->prepare(
						'ALTER TABLE %i ADD CONSTRAINT `fk_icon_id` FOREIGN KEY (`icon_id`) REFERENCES %i(`id`) ON DELETE CASCADE ON UPDATE CASCADE',
						self::get_onlinemap_locations_table_name(),
						self::get_icons_table_name()
					)
				);
			}
		}
	}
}
